@@ -27,6 +27,7 @@ from oslo_config import cfg
 from oslo_db import options as db_options
 from oslo_log import log as logging
 import oslo_messaging as messaging
+from oslo_policy import opts as policy_opts
 
 from octavia.certificates.common import local
 from octavia.common import constants
@@ -133,10 +134,17 @@ api_opts = [
                choices=constants.TLS_ALL_VERSIONS + [None],
                help=_('Minimum allowed TLS version for listeners and pools.')),
     cfg.ListOpt('default_listener_alpn_protocols',
-                default=[lib_consts.ALPN_PROTOCOL_HTTP_1_1,
+                default=[lib_consts.ALPN_PROTOCOL_HTTP_2,
+                         lib_consts.ALPN_PROTOCOL_HTTP_1_1,
                          lib_consts.ALPN_PROTOCOL_HTTP_1_0],
                 help=_('List of ALPN protocols to use for new TLS-enabled '
                        'listeners.')),
+    cfg.ListOpt('default_pool_alpn_protocols',
+                default=[lib_consts.ALPN_PROTOCOL_HTTP_2,
+                         lib_consts.ALPN_PROTOCOL_HTTP_1_1,
+                         lib_consts.ALPN_PROTOCOL_HTTP_1_0],
+                help=_('List of ALPN protocols to use for new TLS-enabled '
+                       'pools.')),
 ]
 
 # Options only used by the amphora agent
@@ -152,7 +160,11 @@ amphora_agent_opts = [
     cfg.StrOpt('agent_server_network_file',
                help=_("The file where the network interfaces are located. "
                       "Specifying this will override any value set for "
-                      "agent_server_network_dir.")),
+                      "agent_server_network_dir."),
+               deprecated_for_removal=True,
+               deprecated_reason=_('New amphora interface management '
+                                   'does not support single interface file.'),
+               deprecated_since='Xena'),
     cfg.IntOpt('agent_request_read_timeout', default=180,
                help=_("The time in seconds to allow a request from the "
                       "controller to run before terminating the socket.")),
@@ -208,7 +220,11 @@ amphora_agent_opts = [
     cfg.StrOpt('amphora_id', help=_("The amphora ID.")),
     cfg.StrOpt('amphora_udp_driver',
                default='keepalived_lvs',
-               help='The UDP API backend for amphora agent.'),
+               help='The UDP API backend for amphora agent.',
+               deprecated_for_removal=True,
+               deprecated_reason=_('amphora-agent will not support any other '
+                                   'backend than keepalived_lvs.'),
+               deprecated_since='Wallaby'),
 ]
 
 compute_opts = [
@@ -605,20 +621,6 @@ certificate_opts = [
 ]
 
 house_keeping_opts = [
-    cfg.IntOpt('spare_check_interval',
-               deprecated_for_removal=True,
-               deprecated_since='Victoria',
-               deprecated_reason='Spares Pool support will be removed in the '
-                                 'X release.',
-               default=30,
-               help=_('Spare check interval in seconds')),
-    cfg.IntOpt('spare_amphora_pool_size',
-               deprecated_for_removal=True,
-               deprecated_since='Victoria',
-               deprecated_reason='Spares Pool support will be removed in the '
-                                 'X release.',
-               default=0,
-               help=_('Number of spare amphorae')),
     cfg.IntOpt('cleanup_interval',
                default=30,
                help=_('DB cleanup interval in seconds')),
@@ -972,3 +974,19 @@ def setup_remote_debugger():
                       {'debug-host': debugger_host,
                        'debug-port': debugger_port})
         raise
+
+
+def set_lib_defaults():
+    """Update default value for configuration options from other namespace.
+
+    Example, oslo lib config options. This is needed for
+    config generator tool to pick these default value changes.
+    https://docs.openstack.org/oslo.config/latest/cli/
+    generator.html#modifying-defaults-from-other-namespaces
+    """
+
+    # TODO(gmann): Remove setting the default value of config policy_file
+    # once oslo_policy change the default value to 'policy.yaml'.
+    # https://github.com/openstack/oslo.policy/blob/a626ad12fe5a3abd49d70e3e5b95589d279ab578/oslo_policy/opts.py#L49
+    # Update default value of oslo.policy policy_file config option.
+    policy_opts.set_defaults(cfg.CONF, 'policy.yaml')
