@@ -100,6 +100,9 @@ class JinjaTemplater(object):
         # Is it newer than haproxy 1.5?
         if not (int(haproxy_versions[0]) < 2 and int(haproxy_versions[1]) < 6):
             feature_compatibility[constants.HTTP_REUSE] = True
+            feature_compatibility[constants.SERVER_STATE_FILE] = True
+        if not (int(haproxy_versions[0]) < 2 and int(haproxy_versions[1]) < 9):
+            feature_compatibility[constants.POOL_ALPN] = True
 
         return self.render_loadbalancer_obj(
             host_amphora, listeners, tls_certs=tls_certs,
@@ -160,11 +163,16 @@ class JinjaTemplater(object):
         if not socket_path:
             socket_path = '%s/%s.sock' % (self.base_amp_path,
                                           listeners[0].load_balancer.id)
+        state_file_path = '%s/%s/servers-state' % (
+            self.base_amp_path,
+            listeners[0].load_balancer.id) if feature_compatibility.get(
+            constants.SERVER_STATE_FILE) else ''
         return self._get_template().render(
             {'loadbalancer': loadbalancer,
              'stats_sock': socket_path,
              'log_http': self.log_http,
              'log_server': self.log_server,
+             'state_file': state_file_path,
              'administrative_log_facility':
                  CONF.amphora_agent.administrative_log_facility,
              'user_log_facility': CONF.amphora_agent.user_log_facility,
@@ -179,7 +187,7 @@ class JinjaTemplater(object):
         """
         listener_transforms = []
         for listener in listeners:
-            if listener.protocol == constants.PROTOCOL_UDP:
+            if listener.protocol in constants.LVS_PROTOCOLS:
                 continue
             listener_transforms.append(self._transform_listener(
                 listener, tls_certs, feature_compatibility, loadbalancer))
@@ -199,7 +207,7 @@ class JinjaTemplater(object):
         # listeners' connection limits.
         connection_limit_sum = 0
         for listener in listeners:
-            if listener.protocol == constants.PROTOCOL_UDP:
+            if listener.protocol in constants.LVS_PROTOCOLS:
                 continue
             if listener.connection_limit and listener.connection_limit > -1:
                 connection_limit_sum += listener.connection_limit
@@ -370,6 +378,9 @@ class JinjaTemplater(object):
                 ret_value['tls_ciphers'] = pool.tls_ciphers
             if pool.tls_versions is not None:
                 ret_value['tls_versions'] = pool.tls_versions
+            if (pool.alpn_protocols is not None and
+                    feature_compatibility.get(constants.POOL_ALPN, False)):
+                ret_value['alpn_protocols'] = ",".join(pool.alpn_protocols)
         if (pool.ca_tls_certificate_id and pool_tls_certs and
                 pool_tls_certs.get('ca_cert')):
             ret_value['ca_cert'] = pool_tls_certs.get('ca_cert')
