@@ -12,12 +12,13 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 #
+from unittest import mock
 
 from cryptography import fernet
-import mock
 from oslo_config import cfg
 from oslo_config import fixture as oslo_fixture
 from oslo_utils import uuidutils
+import tenacity
 
 from octavia.common import constants
 from octavia.common import exceptions
@@ -28,7 +29,6 @@ import octavia.tests.unit.base as base
 
 
 AMP_FLAVOR_ID = '10'
-AMP_IMAGE_ID = '11'
 AMP_IMAGE_TAG = 'glance_tag'
 AMP_SSH_KEY_NAME = None
 AMP_NET = [uuidutils.generate_uuid()]
@@ -65,8 +65,6 @@ class TestComputeTasks(base.TestCase):
         self.conf = self.useFixture(oslo_fixture.Config(cfg.CONF))
         self.conf.config(
             group="controller_worker", amp_flavor_id=AMP_FLAVOR_ID)
-        self.conf.config(
-            group="controller_worker", amp_image_id=AMP_IMAGE_ID)
         self.conf.config(
             group="controller_worker", amp_image_tag=AMP_IMAGE_TAG)
         self.conf.config(
@@ -85,7 +83,7 @@ class TestComputeTasks(base.TestCase):
         logging_mock = mock.MagicMock()
         compute_tasks.LOG = logging_mock
 
-        super(TestComputeTasks, self).setUp()
+        super().setUp()
 
     @mock.patch('octavia.common.jinja.logging.logging_jinja_cfg.'
                 'LoggingJinjaTemplater.build_logging_config')
@@ -93,9 +91,12 @@ class TestComputeTasks(base.TestCase):
     @mock.patch('octavia.amphorae.backends.agent.'
                 'agent_jinja_cfg.AgentJinjaTemplater.'
                 'build_agent_config', return_value='test_conf')
+    @mock.patch('octavia.common.jinja.'
+                'user_data_jinja_cfg.UserDataJinjaCfg.'
+                'build_user_data_config', return_value='user_data_conf')
     @mock.patch('stevedore.driver.DriverManager.driver')
-    def test_compute_create(self, mock_driver, mock_conf, mock_jinja,
-                            mock_log_cfg):
+    def test_compute_create(self, mock_driver, mock_ud_conf,
+                            mock_conf, mock_jinja, mock_log_cfg):
 
         image_owner_id = uuidutils.generate_uuid()
         self.conf.config(
@@ -113,7 +114,6 @@ class TestComputeTasks(base.TestCase):
         mock_driver.build.assert_called_once_with(
             name="amphora-" + _amphora_mock.id,
             amphora_flavor=AMP_FLAVOR_ID,
-            image_id=AMP_IMAGE_ID,
             image_tag=AMP_IMAGE_TAG,
             image_owner=image_owner_id,
             key_name=AMP_SSH_KEY_NAME,
@@ -123,8 +123,9 @@ class TestComputeTasks(base.TestCase):
             config_drive_files={'/etc/octavia/'
                                 'amphora-agent.conf': 'test_conf',
                                 '/etc/rsyslog.d/10-rsyslog.conf': 'FAKE CFG'},
-            user_data=None,
-            server_group_id=SERVER_GRPOUP_ID)
+            user_data='user_data_conf',
+            server_group_id=SERVER_GRPOUP_ID,
+            availability_zone=None)
 
         # Make sure it returns the expected compute_id
         self.assertEqual(COMPUTE_ID, compute_id)
@@ -157,25 +158,24 @@ class TestComputeTasks(base.TestCase):
                 'build_agent_config', return_value='test_conf')
     @mock.patch('octavia.common.jinja.'
                 'user_data_jinja_cfg.UserDataJinjaCfg.'
-                'build_user_data_config', return_value='test_conf')
+                'build_user_data_config', return_value='user_data_conf')
     @mock.patch('stevedore.driver.DriverManager.driver')
     def test_compute_create_user_data(self, mock_driver,
                                       mock_ud_conf, mock_conf, mock_jinja):
 
         self.conf.config(
             group="controller_worker", user_data_config_drive=True)
-        mock_ud_conf.return_value = 'test_ud_conf'
         createcompute = compute_tasks.ComputeCreate()
 
         mock_driver.build.return_value = COMPUTE_ID
         # Test execute()
-        compute_id = createcompute.execute(_amphora_mock.id, ports=[_port])
+        compute_id = createcompute.execute(_amphora_mock.id, ports=[_port],
+                                           server_group_id=None)
 
         # Validate that the build method was called properly
         mock_driver.build.assert_called_once_with(
             name="amphora-" + _amphora_mock.id,
             amphora_flavor=AMP_FLAVOR_ID,
-            image_id=AMP_IMAGE_ID,
             image_tag=AMP_IMAGE_TAG,
             image_owner='',
             key_name=AMP_SSH_KEY_NAME,
@@ -183,8 +183,9 @@ class TestComputeTasks(base.TestCase):
             network_ids=AMP_NET,
             port_ids=[PORT_ID],
             config_drive_files=None,
-            user_data='test_ud_conf',
-            server_group_id=None)
+            user_data='user_data_conf',
+            server_group_id=None,
+            availability_zone=None)
 
         # Make sure it returns the expected compute_id
         self.assertEqual(COMPUTE_ID, compute_id)
@@ -217,15 +218,89 @@ class TestComputeTasks(base.TestCase):
     @mock.patch('octavia.amphorae.backends.agent.'
                 'agent_jinja_cfg.AgentJinjaTemplater.'
                 'build_agent_config', return_value='test_conf')
+    @mock.patch('octavia.common.jinja.'
+                'user_data_jinja_cfg.UserDataJinjaCfg.'
+                'build_user_data_config', return_value='user_data_conf')
     @mock.patch('stevedore.driver.DriverManager.driver')
+    def test_compute_create_availability_zone(self, mock_driver, mock_ud_conf,
+                                              mock_conf, mock_jinja,
+                                              mock_log_cfg):
+
+        image_owner_id = uuidutils.generate_uuid()
+        compute_zone = uuidutils.generate_uuid()
+        az_dict = {constants.COMPUTE_ZONE: compute_zone}
+
+        self.conf.config(
+            group="controller_worker", amp_image_owner_id=image_owner_id)
+        mock_log_cfg.return_value = 'FAKE CFG'
+
+        createcompute = compute_tasks.ComputeCreate()
+
+        mock_driver.build.return_value = COMPUTE_ID
+        # Test execute()
+        compute_id = createcompute.execute(_amphora_mock.id, ports=[_port],
+                                           server_group_id=SERVER_GRPOUP_ID,
+                                           availability_zone=az_dict)
+
+        # Validate that the build method was called properly
+        mock_driver.build.assert_called_once_with(
+            name="amphora-" + _amphora_mock.id,
+            amphora_flavor=AMP_FLAVOR_ID,
+            image_tag=AMP_IMAGE_TAG,
+            image_owner=image_owner_id,
+            key_name=AMP_SSH_KEY_NAME,
+            sec_groups=AMP_SEC_GROUPS,
+            network_ids=AMP_NET,
+            port_ids=[PORT_ID],
+            config_drive_files={'/etc/octavia/'
+                                'amphora-agent.conf': 'test_conf',
+                                '/etc/rsyslog.d/10-rsyslog.conf': 'FAKE CFG'},
+            user_data='user_data_conf',
+            server_group_id=SERVER_GRPOUP_ID,
+            availability_zone=compute_zone)
+
+        # Make sure it returns the expected compute_id
+        self.assertEqual(COMPUTE_ID, compute_id)
+
+        # Test that a build exception is raised
+        createcompute = compute_tasks.ComputeCreate()
+
+        self.assertRaises(TypeError,
+                          createcompute.execute,
+                          _amphora_mock, config_drive_files='test_cert')
+
+        # Test revert()
+
+        _amphora_mock.compute_id = COMPUTE_ID
+
+        createcompute = compute_tasks.ComputeCreate()
+        createcompute.revert(compute_id, _amphora_mock.id)
+
+        # Validate that the delete method was called properly
+        mock_driver.delete.assert_called_once_with(
+            COMPUTE_ID)
+
+        # Test that a delete exception is not raised
+
+        createcompute.revert(COMPUTE_ID, _amphora_mock.id)
+
+    @mock.patch('octavia.common.jinja.logging.logging_jinja_cfg.'
+                'LoggingJinjaTemplater.build_logging_config')
+    @mock.patch('jinja2.Environment.get_template')
+    @mock.patch('octavia.amphorae.backends.agent.'
+                'agent_jinja_cfg.AgentJinjaTemplater.'
+                'build_agent_config', return_value='test_conf')
+    @mock.patch('octavia.common.jinja.'
+                'user_data_jinja_cfg.UserDataJinjaCfg.'
+                'build_user_data_config', return_value='user_data_conf')
+    @mock.patch('stevedore.driver.DriverManager.driver')
     def test_compute_create_without_ssh_access(
-            self, mock_driver, mock_conf, mock_jinja, mock_log_cfg):
+            self, mock_driver, mock_user_data_config,
+            mock_conf, mock_jinja, mock_log_cfg):
 
         createcompute = compute_tasks.ComputeCreate()
 
         mock_driver.build.return_value = COMPUTE_ID
-        self.conf.config(
-            group="controller_worker", amp_ssh_access_allowed=False)
         self.conf.config(
             group="controller_worker", user_data_config_drive=False)
         mock_log_cfg.return_value = 'FAKE CFG'
@@ -238,7 +313,6 @@ class TestComputeTasks(base.TestCase):
         mock_driver.build.assert_called_once_with(
             name="amphora-" + _amphora_mock.id,
             amphora_flavor=AMP_FLAVOR_ID,
-            image_id=AMP_IMAGE_ID,
             image_tag=AMP_IMAGE_TAG,
             image_owner='',
             key_name=None,
@@ -248,8 +322,9 @@ class TestComputeTasks(base.TestCase):
             config_drive_files={'/etc/octavia/'
                                 'amphora-agent.conf': 'test_conf',
                                 '/etc/rsyslog.d/10-rsyslog.conf': 'FAKE CFG'},
-            user_data=None,
-            server_group_id=SERVER_GRPOUP_ID)
+            user_data='user_data_conf',
+            server_group_id=SERVER_GRPOUP_ID,
+            availability_zone=None)
 
         self.assertEqual(COMPUTE_ID, compute_id)
 
@@ -281,11 +356,14 @@ class TestComputeTasks(base.TestCase):
     @mock.patch('octavia.amphorae.backends.agent.'
                 'agent_jinja_cfg.AgentJinjaTemplater.'
                 'build_agent_config', return_value='test_conf')
+    @mock.patch('octavia.common.jinja.'
+                'user_data_jinja_cfg.UserDataJinjaCfg.'
+                'build_user_data_config', return_value='user_data_conf')
     @mock.patch('stevedore.driver.DriverManager.driver')
-    def test_compute_create_cert(self, mock_driver, mock_conf, mock_jinja,
-                                 mock_log_cfg):
+    def test_compute_create_cert(self, mock_driver, mock_ud_conf,
+                                 mock_conf, mock_jinja, mock_log_cfg):
         createcompute = compute_tasks.CertComputeCreate()
-        key = utils.get_six_compatible_server_certs_key_passphrase()
+        key = utils.get_compatible_server_certs_key_passphrase()
         fer = fernet.Fernet(key)
         mock_log_cfg.return_value = 'FAKE CFG'
 
@@ -294,7 +372,7 @@ class TestComputeTasks(base.TestCase):
         self.useFixture(test_utils.OpenFixture(path, 'test'))
         # Test execute()
         test_cert = fer.encrypt(
-            utils.get_six_compatible_value('test_cert')
+            utils.get_compatible_value('test_cert')
         )
         compute_id = createcompute.execute(_amphora_mock.id, test_cert,
                                            server_group_id=SERVER_GRPOUP_ID
@@ -304,20 +382,20 @@ class TestComputeTasks(base.TestCase):
         mock_driver.build.assert_called_once_with(
             name="amphora-" + _amphora_mock.id,
             amphora_flavor=AMP_FLAVOR_ID,
-            image_id=AMP_IMAGE_ID,
             image_tag=AMP_IMAGE_TAG,
             image_owner='',
             key_name=AMP_SSH_KEY_NAME,
             sec_groups=AMP_SEC_GROUPS,
             network_ids=AMP_NET,
             port_ids=[],
-            user_data=None,
+            user_data='user_data_conf',
             config_drive_files={
                 '/etc/rsyslog.d/10-rsyslog.conf': 'FAKE CFG',
                 '/etc/octavia/certs/server.pem': fer.decrypt(test_cert),
                 '/etc/octavia/certs/client_ca.pem': 'test',
                 '/etc/octavia/amphora-agent.conf': 'test_conf'},
-            server_group_id=SERVER_GRPOUP_ID)
+            server_group_id=SERVER_GRPOUP_ID,
+            availability_zone=None)
 
         self.assertEqual(COMPUTE_ID, compute_id)
 
@@ -361,15 +439,23 @@ class TestComputeTasks(base.TestCase):
         mock_driver.get_amphora.return_value = _amphora_mock, None
 
         computewait = compute_tasks.ComputeActiveWait()
-        computewait.execute(COMPUTE_ID, AMPHORA_ID)
 
-        mock_driver.get_amphora.assert_called_once_with(COMPUTE_ID)
+        # Test with no AZ
+        computewait.execute(COMPUTE_ID, AMPHORA_ID, None)
+        mock_driver.get_amphora.assert_called_once_with(COMPUTE_ID, None)
 
+        # Test with AZ
+        mock_driver.reset_mock()
+        az = {constants.MANAGEMENT_NETWORK: uuidutils.generate_uuid()}
+        computewait.execute(COMPUTE_ID, AMPHORA_ID, az)
+        mock_driver.get_amphora.assert_called_once_with(
+            COMPUTE_ID, az[constants.MANAGEMENT_NETWORK])
+
+        # Test with deleted amp
         _amphora_mock.status = constants.DELETED
-
         self.assertRaises(exceptions.ComputeWaitTimeoutException,
                           computewait.execute,
-                          _amphora_mock, AMPHORA_ID)
+                          _amphora_mock, AMPHORA_ID, None)
 
     @mock.patch('octavia.controller.worker.amphora_rate_limit'
                 '.AmphoraBuildRateLimit.remove_from_build_req_queue')
@@ -388,15 +474,15 @@ class TestComputeTasks(base.TestCase):
         mock_driver.get_amphora.return_value = _amphora_mock, None
 
         computewait = compute_tasks.ComputeActiveWait()
-        computewait.execute(COMPUTE_ID, AMPHORA_ID)
+        computewait.execute(COMPUTE_ID, AMPHORA_ID, None)
 
-        mock_driver.get_amphora.assert_called_once_with(COMPUTE_ID)
+        mock_driver.get_amphora.assert_called_once_with(COMPUTE_ID, None)
 
         _amphora_mock.status = constants.ERROR
 
         self.assertRaises(exceptions.ComputeBuildException,
                           computewait.execute,
-                          _amphora_mock, AMPHORA_ID)
+                          _amphora_mock, AMPHORA_ID, None)
 
     @mock.patch('octavia.controller.worker.amphora_rate_limit'
                 '.AmphoraBuildRateLimit.remove_from_build_req_queue')
@@ -413,27 +499,62 @@ class TestComputeTasks(base.TestCase):
         mock_driver.get_amphora.return_value = _amphora_mock, None
 
         computewait = compute_tasks.ComputeActiveWait()
-        computewait.execute(COMPUTE_ID, AMPHORA_ID)
+        computewait.execute(COMPUTE_ID, AMPHORA_ID, None)
 
-        mock_driver.get_amphora.assert_called_once_with(COMPUTE_ID)
+        mock_driver.get_amphora.assert_called_once_with(COMPUTE_ID, None)
         mock_remove_from_build_queue.assert_not_called()
 
     @mock.patch('stevedore.driver.DriverManager.driver')
     def test_delete_amphorae_on_load_balancer(self, mock_driver):
 
+        mock_driver.delete.side_effect = [mock.DEFAULT,
+                                          exceptions.OctaviaException('boom')]
+
         delete_amps = compute_tasks.DeleteAmphoraeOnLoadBalancer()
+
         delete_amps.execute(_load_balancer_mock)
 
         mock_driver.delete.assert_called_once_with(COMPUTE_ID)
 
+        # Test compute driver exception is raised
+        self.assertRaises(exceptions.OctaviaException, delete_amps.execute,
+                          _load_balancer_mock)
+
     @mock.patch('stevedore.driver.DriverManager.driver')
     def test_compute_delete(self, mock_driver):
+        mock_driver.delete.side_effect = [
+            mock.DEFAULT, exceptions.OctaviaException('boom'),
+            mock.DEFAULT, exceptions.OctaviaException('boom'),
+            exceptions.OctaviaException('boom'),
+            exceptions.OctaviaException('boom'),
+            exceptions.OctaviaException('boom')]
 
         delete_compute = compute_tasks.ComputeDelete()
+
+        # Limit the retry attempts for the test run to save time
+        delete_compute.execute.retry.stop = tenacity.stop_after_attempt(2)
+
         delete_compute.execute(_amphora_mock)
 
         mock_driver.delete.assert_called_once_with(COMPUTE_ID)
 
+        # Test retry after a compute exception
+        mock_driver.reset_mock()
+        delete_compute.execute(_amphora_mock)
+        mock_driver.delete.assert_has_calls([mock.call(COMPUTE_ID),
+                                            mock.call(COMPUTE_ID)])
+
+        # Test passive failure
+        mock_driver.reset_mock()
+        delete_compute.execute(_amphora_mock, passive_failure=True)
+        mock_driver.delete.assert_has_calls([mock.call(COMPUTE_ID),
+                                            mock.call(COMPUTE_ID)])
+
+        # Test non-passive failure
+        mock_driver.reset_mock()
+        self.assertRaises(exceptions.OctaviaException, delete_compute.execute,
+                          _amphora_mock, passive_failure=False)
+
     @mock.patch('stevedore.driver.DriverManager.driver')
     def test_nova_server_group_create(self, mock_driver):
         nova_sever_group_obj = compute_tasks.NovaServerGroupCreate()
@@ -479,3 +600,34 @@ class TestComputeTasks(base.TestCase):
         sg_id = None
         nova_sever_group_obj.execute(sg_id)
         self.assertFalse(mock_driver.delete_server_group.called, sg_id)
+
+    @mock.patch('stevedore.driver.DriverManager.driver')
+    def test_attach_port(self, mock_driver):
+        COMPUTE_ID = uuidutils.generate_uuid()
+        PORT_ID = uuidutils.generate_uuid()
+        amphora_mock = mock.MagicMock()
+        port_mock = mock.MagicMock()
+        amphora_mock.compute_id = COMPUTE_ID
+        port_mock.id = PORT_ID
+
+        attach_port_obj = compute_tasks.AttachPort()
+
+        # Test execute
+        attach_port_obj.execute(amphora_mock, port_mock)
+
+        mock_driver.attach_network_or_port.assert_called_once_with(
+            COMPUTE_ID, port_id=PORT_ID)
+
+        # Test revert
+        mock_driver.reset_mock()
+
+        attach_port_obj.revert(amphora_mock, port_mock)
+
+        mock_driver.detach_port.assert_called_once_with(COMPUTE_ID, PORT_ID)
+
+        # Test rever exception
+        mock_driver.reset_mock()
+        mock_driver.detach_port.side_effect = [Exception('boom')]
+
+        # should not raise
+        attach_port_obj.revert(amphora_mock, port_mock)
