@@ -210,8 +210,14 @@ class TestLoadBalancerFlows(base.TestCase):
         self.assertIsInstance(create_flow, flow.Flow)
         self.assertIn(constants.LOADBALANCER_ID, create_flow.requires)
         self.assertIn(constants.UPDATE_DICT, create_flow.requires)
+        self.assertIn(constants.BUILD_TYPE_PRIORITY, create_flow.requires)
+        self.assertIn(constants.FLAVOR, create_flow.requires)
+        self.assertIn(constants.AMPHORAE_NETWORK_CONFIG, create_flow.requires)
+        self.assertIn(constants.AVAILABILITY_ZONE, create_flow.requires)
+        self.assertIn(constants.SERVER_GROUP_ID, create_flow.requires)
 
         self.assertIn(constants.LISTENERS, create_flow.provides)
+        self.assertIn(constants.SUBNET, create_flow.provides)
         self.assertIn(constants.AMPHORA, create_flow.provides)
         self.assertIn(constants.AMPHORA_ID, create_flow.provides)
         self.assertIn(constants.AMPHORA_NETWORK_CONFIG, create_flow.provides)
@@ -220,12 +226,11 @@ class TestLoadBalancerFlows(base.TestCase):
         self.assertIn(constants.COMPUTE_OBJ, create_flow.provides)
         self.assertIn(constants.LOADBALANCER, create_flow.provides)
         self.assertIn(constants.DELTAS, create_flow.provides)
-        self.assertIn(constants.ADDED_PORTS, create_flow.provides)
+        self.assertIn(constants.UPDATED_PORTS, create_flow.provides)
         self.assertIn(constants.SERVER_PEM, create_flow.provides)
-        self.assertIn(constants.SUBNET, create_flow.provides)
         self.assertIn(constants.VIP, create_flow.provides)
 
-        self.assertEqual(6, len(create_flow.requires))
+        self.assertEqual(7, len(create_flow.requires))
         self.assertEqual(13, len(create_flow.provides),
                          create_flow.provides)
 
@@ -244,7 +249,7 @@ class TestLoadBalancerFlows(base.TestCase):
         self.assertIn(constants.SERVER_GROUP_ID, create_flow.requires)
         self.assertIn(constants.UPDATE_DICT, create_flow.requires)
 
-        self.assertIn(constants.ADDED_PORTS, create_flow.provides)
+        self.assertIn(constants.UPDATED_PORTS, create_flow.provides)
         self.assertIn(constants.AMP_DATA, create_flow.provides)
         self.assertIn(constants.AMP_VRRP_INT, create_flow.provides)
         self.assertIn(constants.AMPHORA, create_flow.provides)
@@ -280,7 +285,7 @@ class TestLoadBalancerFlows(base.TestCase):
         self.assertIn(constants.LOADBALANCER, failover_flow.requires)
         self.assertIn(constants.LOADBALANCER_ID, failover_flow.requires)
 
-        self.assertIn(constants.ADDED_PORTS, failover_flow.provides)
+        self.assertIn(constants.UPDATED_PORTS, failover_flow.provides)
         self.assertIn(constants.AMPHORA, failover_flow.provides)
         self.assertIn(constants.AMPHORA_ID, failover_flow.provides)
         self.assertIn(constants.AMPHORAE_NETWORK_CONFIG,
@@ -310,15 +315,6 @@ class TestLoadBalancerFlows(base.TestCase):
 
         self._test_get_failover_LB_flow_single([amphora_dict])
 
-    def test_get_failover_LB_flow_one_spare_amp_single(self,
-                                                       mock_get_net_driver):
-        amphora_dict = {constants.ID: uuidutils.generate_uuid(),
-                        constants.ROLE: 'bogus',
-                        constants.COMPUTE_ID: uuidutils.generate_uuid(),
-                        constants.VRRP_PORT_ID: None, constants.VRRP_IP: None}
-
-        self._test_get_failover_LB_flow_single([amphora_dict])
-
     def test_get_failover_LB_flow_one_bogus_amp_single(self,
                                                        mock_get_net_driver):
         amphora_dict = {constants.ID: uuidutils.generate_uuid(),
@@ -352,7 +348,7 @@ class TestLoadBalancerFlows(base.TestCase):
         self.assertIn(constants.LOADBALANCER, failover_flow.requires)
         self.assertIn(constants.LOADBALANCER_ID, failover_flow.requires)
 
-        self.assertIn(constants.ADDED_PORTS, failover_flow.provides)
+        self.assertIn(constants.UPDATED_PORTS, failover_flow.provides)
         self.assertIn(constants.AMPHORA, failover_flow.provides)
         self.assertIn(constants.AMPHORA_ID, failover_flow.provides)
         self.assertIn(constants.AMPHORAE_NETWORK_CONFIG,
@@ -434,22 +430,6 @@ class TestLoadBalancerFlows(base.TestCase):
         self._test_get_failover_LB_flow_no_amps_act_stdby([amphora_dict,
                                                            amphora2_dict])
 
-    def test_get_failover_LB_flow_two_amps_spare_act_stdby(
-            self, mock_get_net_driver):
-        amphora_dict = {constants.ID: uuidutils.generate_uuid(),
-                        constants.ROLE: None,
-                        constants.COMPUTE_ID: uuidutils.generate_uuid(),
-                        constants.VRRP_PORT_ID: uuidutils.generate_uuid(),
-                        constants.VRRP_IP: '192.0.2.46'}
-        amphora2_dict = {constants.ID: uuidutils.generate_uuid(),
-                         constants.ROLE: constants.ROLE_MASTER,
-                         constants.COMPUTE_ID: uuidutils.generate_uuid(),
-                         constants.VRRP_PORT_ID: uuidutils.generate_uuid(),
-                         constants.VRRP_IP: '2001:db8::46'}
-
-        self._test_get_failover_LB_flow_no_amps_act_stdby([amphora_dict,
-                                                           amphora2_dict])
-
     def test_get_failover_LB_flow_two_amps_standalone_act_stdby(
             self, mock_get_net_driver):
         amphora_dict = {constants.ID: uuidutils.generate_uuid(),
